@@ -1,17 +1,19 @@
 import re
 
 from dataclasses import dataclass
-from typing import Optional, Tuple, Any, List
+from typing import Optional, Tuple, Any, List, Union
 
 @dataclass
 class Instruction:
     text_line_nr: int
     mnemonic: str
-    args: Optional[Tuple[str]] = None
+    arglist: Optional[Tuple[str]] = None
     label: Optional[str] = None
+    args: List[Union[int,str]] = None
     reg_args: List[int] = None
     func_name: Optional[str] = None
     func: Any = None
+    clock_ticks: int = 1
 
 
 class AsmSyntaxError(Exception):
@@ -42,10 +44,13 @@ mnemonic_args = {
     'set_ph_delta': 'IR',
     'set_awg_gain': 'IR,IR',
     'set_awg_offs': 'IR,IR',
+    'set_cond': 'IR,IR,IR,I',
     'upd_param': 'I',
     'play': 'IR,IR,I',
     'acquire': 'I,IR,I',
     'acquire_weighed': 'I,IR,IR,IR,I',
+    'set_latch_en': 'IR,I',
+    'latch_rst': 'IR',
     'wait': 'IR',
     'wait_sync': 'IR',
     'wait_trigger': 'IR',
@@ -82,17 +87,24 @@ class Q1Parser:
             instr.func_name = func_name
             if mnemonic in mnemonic_args:
                 try:
-                    args,reg_args = self._evaluate_args(mnemonic_args[mnemonic], instr.args)
+                    args,reg_args = self._evaluate_args(mnemonic_args[mnemonic], instr.arglist)
                     instr.args = args
                     instr.reg_args = reg_args
+                    if reg_args and mnemonic not in ['jmp', 'jge', 'jlt', 'loop']:
+                        # 1 cycle for every register. instr and 1st register are loaded in 1 cycle
+                        instr.clock_ticks = len(reg_args)
+                    else:
+                        # 1 cycle to load instruction
+                        instr.clock_ticks = 1
                 except AsmSyntaxError as ex:
                     print(ex)
                     print(lines[instr.text_line_nr])
                     raise
+            else:
+                instr.args = instr.arglist
 
         return lines,instructions
 
-
     def _parseline(self, line):
         org_line = line
         label_pattern = r'(\w+:)'
@@ -121,7 +133,7 @@ class Q1Parser:
                 label = label[:-1]
             args = match.group(3).strip()
             if args:
-                arglist = args.split(',')
+                arglist = args.replace(' ','').split(',')
             else:
                 arglist = []
             return [label, match.group(2), arglist]
@@ -139,6 +151,12 @@ class Q1Parser:
             if msg is None:
                 msg = ''
             return None,'log',(msg,register,options)
+        trigger_pattern = r'sim_trigger (\d),\s*([01])'
+        match = re.fullmatch(trigger_pattern, command)
+        if match:
+            addr = match.group(1)
+            value = match.group(2)
+            return None,'sim_trigger',(addr, value)
         print(f'Unknown simulator command:{command}')
         return None,None,None
 
