@@ -7,6 +7,8 @@ import qcodes as qc
 
 from .q1sequencer import Q1Sequencer
 from .qblox_version import check_qblox_instrument_version
+from .triggers import TriggerDistributor
+from .trigger_sorting import get_seq_trigger_info, sort_sequencers
 
 from qblox_instruments import (
         SystemStatus, SystemState, SystemStatusSlotFlags,
@@ -15,16 +17,24 @@ from qblox_instruments import (
 
 logger = logging.getLogger(__name__)
 
+
 class Q1Module(qc.instrument.InstrumentBase):
+    _module_parameters = [
+        'ext_trigger_input_delay',
+        'ext_trigger_input_trigger_en',
+        'ext_trigger_input_trigger_address',
+        'marker0_inv_en',
+        'marker1_inv_en',
+        'marker2_inv_en',
+        'marker3_inv_en',
+        ]
     _sim_parameters_qcm = [
-        'reference_source',
         'out0_offset',
         'out1_offset',
         'out2_offset',
         'out3_offset',
         ]
     _sim_parameters_qcm_rf = [
-        'reference_source',
         'out0_lo_freq',
         'out1_lo_freq',
         'out0_lo_en',
@@ -37,7 +47,6 @@ class Q1Module(qc.instrument.InstrumentBase):
         'out1_offset_path1',
         ]
     _sim_parameters_qrm = [
-        'reference_source',
         'out0_offset',
         'out1_offset',
         'in0_gain',
@@ -49,11 +58,14 @@ class Q1Module(qc.instrument.InstrumentBase):
         'scope_acq_sequencer_select',
         'scope_acq_avg_mode_en_path0',
         'scope_acq_avg_mode_en_path1',
+        'in0_offset',
+        'in1_offset',
         ]
     _sim_parameters_qrm_rf = [
-        'reference_source',
         'in0_att',
         'out0_att',
+        'in0_offset_path0',
+        'in0_offset_path1',
         'out0_in0_lo_freq',
         'out0_in0_lo_en',
         'out0_offset_path0',
@@ -71,7 +83,6 @@ class Q1Module(qc.instrument.InstrumentBase):
     # This class is used as a mixin. Although quite heavy mixin.
 
     def init_module(self, n_sequencers=6, sim_type=None):
-        check_qblox_instrument_version()
         self._sim_type = sim_type
         if sim_type is None:
             raise Exception('sim_type must be specified')
@@ -83,23 +94,25 @@ class Q1Module(qc.instrument.InstrumentBase):
         if not (self._is_qcm or self._is_qrm):
             raise ValueError(f'Unknown sim_type: {sim_type}')
 
+        sim_params = []
+        sim_params += self._module_parameters
         if sim_type == 'QCM':
-            sim_params = self._sim_parameters_qcm
+            sim_params += self._sim_parameters_qcm
         elif sim_type == 'QCM-RF':
-            sim_params = self._sim_parameters_qcm_rf
+            sim_params += self._sim_parameters_qcm_rf
         elif sim_type == 'QRM':
-            sim_params = self._sim_parameters_qrm
+            sim_params += self._sim_parameters_qrm
         elif sim_type == 'QRM-RF':
             sim_params = self._sim_parameters_qrm_rf
-        else:
-            sim_params = []
+        elif sim_type == 'Viewer':
+            sim_params += list(set(self._sim_parameters_qcm+self._sim_parameters_qrm))
 
         for par_name in sim_params:
             self.add_parameter(par_name, set_cmd=partial(self._set, par_name))
 
         self.sequencers = [Q1Sequencer(self, f'seq{i}', sim_type)
                            for i in range(n_sequencers)]
-        for i,seq in enumerate(self.sequencers):
+        for i, seq in enumerate(self.sequencers):
             self.add_submodule(f'sequencer{i}', seq)
 
         self.armed_seq = set()
@@ -186,10 +199,12 @@ class Q1Module(qc.instrument.InstrumentBase):
 
     def plot(self, **kwargs):
         for seq in self.sequencers:
-            seq.plot()
+            # assume only sequencers in sync mode have executed.
+            if seq.sync_en():
+                seq.plot()
 
     def print_acquisitions(self):
-        for i,seq in enumerate(self.sequencers):
+        for i, seq in enumerate(self.sequencers):
             data = self.get_acquisitions(i)
             if not len(data):
                 continue
@@ -199,29 +214,47 @@ class Q1Module(qc.instrument.InstrumentBase):
 
                 print("  'path0': [",
                       np.array2string(np.array(bins['integration']['path0']),
-                                        prefix=' '*12,
-                                        separator=',',
-                                        threshold=100),']')
+                                      prefix=' '*12,
+                                      separator=',',
+                                      threshold=100),
+                      ']')
                 print("  'path1': [",
                       np.array2string(np.array(bins['integration']['path1']),
                                       prefix=' '*12,
-                                        separator=',',
-                                        threshold=100),']')
+                                      separator=',',
+                                      threshold=100),
+                      ']')
                 print("  'avg_cnt': [",
                       np.array2string(np.array(bins['avg_cnt']),
-                                        prefix=' '*12,
-                                        separator=',',
-                                        threshold=100),']')
+                                      prefix=' '*12,
+                                      separator=',',
+                                      threshold=100),
+                      ']')
 
     def print_registers(self, seq_nr, reg_nrs=None):
         self.sequencers[seq_nr].print_registers(reg_nrs)
 
 
 class Q1Simulator(qc.Instrument, Q1Module):
+    _pulsar_parameters = [
+        'reference_source',
+        ]
+    _log_only_params = [
+        'led_brightness',
+        ]
+
     def __init__(self, name, n_sequencers=6, sim_type=None):
+        check_qblox_instrument_version()
         super().__init__(name)
         super().init_module(n_sequencers, sim_type)
 
+        for par_name in self._pulsar_parameters:
+            self.add_parameter(par_name, set_cmd=partial(self._set, par_name))
+
+        for par_name in self._log_only_params:
+            self.add_parameter(par_name,
+                               set_cmd=partial(self._log_set, par_name))
+
     def get_idn(self):
         return dict(vendor='Q1Simulator', model=self._sim_type, serial='', firmware='')
 
@@ -233,4 +266,26 @@ class Q1Simulator(qc.Instrument, Q1Module):
     def instrument_type(self):
         return InstrumentType[self._sim_type]
 
+    def start_sequencer(self, sequencer: Optional[int] = None):
+        if sequencer is not None:
+            self.sequencers[sequencer].run()
+            return
 
+        # collect sequencers and sort on used triggers
+        # TODO Refactor trigger distribution in runtime distribution.
+        seq_infos = []
+        for seq_number in self.armed_seq:
+            seq_infos.append(
+                    get_seq_trigger_info(self, seq_number, self.sequencers[seq_number])
+                    )
+
+        trigger_dist = TriggerDistributor()
+        seq_infos = sort_sequencers(seq_infos)
+        for seq_info in seq_infos:
+            seq = seq_info.module.sequencers[seq_info.seq_number]
+            seq.set_trigger_events(trigger_dist.get_trigger_events())
+            seq.run()
+            trigger_dist.add_emitted_triggers(seq.get_acq_trigger_events())
+
+    def _log_set(self, name, value):
+        logger.info(f'{self.name}: {name}={value}')
